@@ -5,9 +5,11 @@ package org.rosuda.REngine;
  The reference semantics works by calling {@link #resolve()} (which in turn uses {@link REngine#resolveReference(REXP)} on itself) whenever any methods are accessed. The implementation is not finalized yat and may change as we approach the JRI interface which is more ameanable to reference-style access. Subclasses are free to implement more efficient implementations. */
 public class REXPReference extends REXP {
 	/** engine which will be used to resolve the reference */
-	REngine eng;
+	protected REngine eng;
 	/** an opaque (optional) handle */
-	Object handle;
+	protected Object handle;
+	/** resolved (cached) object */
+	protected REXP resolvedValue;
 
 	/** create an external REXP reference using given engine and handle. The handle value is just an (optional) identifier not used by the implementation directly. */
 	public REXPReference(REngine eng, Object handle) {
@@ -15,17 +17,40 @@ public class REXPReference extends REXP {
 		this.eng = eng;
 		this.handle = handle;
 	}
-	
-	/** resolve the external REXP reference into an actual REXP object. */
+
+	/** shortcut for <code>REXPReference(eng, new Long(handle))</code> that is used by native code */
+	REXPReference(REngine eng, long handle) {
+		this(eng, new Long(handle));
+	}
+
+	/** resolve the external REXP reference into an actual REXP object. In addition, the value (if not <code>null</code>) will be cached for subsequent calls to <code>resolve</code> until <code>invalidate</code> is called. */
 	public REXP resolve() {
+		if (resolvedValue != null)
+			return resolvedValue;
 		try {
-			return eng.resolveReference(this);
+			resolvedValue = eng.resolveReference(this);
+			return resolvedValue;
+		} catch (REXPMismatchException me) {
+			// this should never happen since we are REXPReference
 		} catch(REngineException ee) {
 			// FIXME: what to we do?
 		}
 		return null;
 	}
 
+	/** invalidates any cached representation of the reference */
+	public void invalidate() {
+		resolvedValue = null;
+	}
+	
+	/** finalization that notifies the engine when a reference gets collected */
+	protected void finalize() throws Throwable {
+		try {
+			eng.finalizeReference(this);
+		} finally {
+			super.finalize();
+		}
+	}	
 	// type checks
 	public boolean isString() { return resolve().isString(); }
 	public boolean isNumeric() { return resolve().isNumeric(); }
@@ -41,6 +66,7 @@ public class REXPReference extends REXP {
 	public boolean isRaw() { return resolve().isRaw(); }
 	public boolean isComplex() { return resolve().isComplex(); }
 	public boolean isRecursive() { return resolve().isRecursive(); }
+	public boolean isReference() { return true; }
 
 	// basic accessor methods
 	public String[] asStrings() throws REXPMismatchException { return resolve().asStrings(); }
@@ -55,6 +81,8 @@ public class REXPReference extends REXP {
 	
 	public Object getHandle() { return handle; }
 	
+	public REngine getEngine() { return eng; }
+
 	public String toString() {
 		return super.toString()+"{eng="+eng+",h="+handle+"}";
 	}
